from passerine.db.metadata.entity import EntityMetadata

class EntityMetadataHelper(object):
    """ Entity Metadata Helper """
    @staticmethod
    def imprint(cls, collection_name, indexes):
        """ Imprint the entity metadata to the class (type)

            :param cls: the entity class
            :type  cls: type
            :param collection_name: the name of the collection (known as table, bucket etc.)
            :type  collection_name: str
            :param indexes: the list of indexes
            :type  indexes: list
        """
        metadata = EntityMetadata()

        metadata.cls             = cls
        metadata.collection_name = collection_name
        metadata.index_list      = indexes

        cls.__tdbm__ = metadata

    @staticmethod
    def extract(cls):
        """ Extract the metadata of the given class

            :param cls: the entity class
            :type  cls: type
            :rtype:     passerine.db.metadata.entity.EntityMetadata
        """
        return cls.__tdbm__

    @staticmethod
    def hasMetadata(cls):
        """ Check if the given class *cls* has a metadata

            :param cls: the entity class
            :type  cls: type
            :rtype:     bool
        """
        return '__tdbm__' in dir(cls)



"""
Wrappers for SQLAlchemy
#######################

:Author: Juti Noppornpitak
"""
from os import getpid
from threading import current_thread
from sqlalchemy import create_engine
from sqlalchemy.exc import OperationalError
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from passerine.common import get_logger

Entity = declarative_base()

class Repository(object):
    """Relational Database Service

    This is compatible with SQLAlchemy 0.7+.

    This service provides basic functionality to interact a relational database.
    It also heavily uses lazy loading for the sake of faster startup and resource
    efficiency.

    :param url: a URL to the database, possibly including location, credential
                and name. The default value is ``sqlite:///:memory:``.
    :type  url: str
    :param echo: the flag to echo database operation. The default value is ``False``
    :type  echo: bool
    """

    def __init__(self, url='sqlite:///:memory:', echo=False):
        self._logger = get_logger('%s.%s' % (__name__, self.__class__.__name__))

        self._echo   = echo
        self._url    = url
        self._engine = None

        self._session_maker      = None
        self._sessions           = {}
        self._batch_reflected    = False
        self._reflected_entities = []

    @property
    def session_key(self):
        return '%s.%s' % (getpid(), current_thread().ident)

    @property
    def engine(self):
        """
        Get the SQLAlchemy engine.

        :rtype: sqlalchemy.engine.base.Engine

        .. note:: With the service, it is not recommended to directly use this method.
        """

        if not self._engine:
            self._engine = create_engine(self._url, echo=self._echo)

        return self._engine

    @property
    def url(self):
        """
        Get the connecting URL.

        :return: the URL
        :rtype: string|unicode
        """
        return self._url

    @url.setter
    def url(self, new_url):
        """
        Set the connecting URL.

        :param new_url: a new URL for the database connection.
        """
        self._url    = new_url
        self._engine = None

    def reflect(self, entity_type=None):
        """
        Create the table if necessary.

        :rtype: True if the reflection is made.
        """
        if not self._batch_reflected and not entity_type:
            self._batch_reflected = True

            Entity.metadata.create_all(self.engine)

            return True

        if self.reflected(entity_type):
            return False

        self._reflected_entities.append(entity_type.__name__)

        try:
            entity_type.__table__.create(self.engine)
        except OperationalError:
            return False

        return True

    def reflected(self, entity_type=None):
        return entity_type.__name__ in self._reflected_entities \
            if entity_type \
            else self._batch_reflected

    @property
    def session(self):
        """
        Get a SQLAlchemy session for the current connection.

        :rtype:  sqlalchemy.orm.session.Session
        :return: a session for the connection.
        """
        engine = self.engine

        if not self._session_maker:
            self._session_maker = sessionmaker(bind=engine)

        session_key = self.session_key

        if session_key not in self._sessions:
            self._logger.info('Create new DB session: %s' % session_key)

            self._sessions[session_key] = self._session_maker()

        return self._sessions[session_key]

    def query(self, *entity_types):
        """
        Retrieve the query object for the given type of entities.

        :param entity_type: the type of entity
        :type entity_type:  list

        :rtype:  sqlalchemy.orm.query.Query
        :return: The query object for the given type of entities.
        """
        for entity_type in entity_types:
            if entity_type.__name__ in self._entity_map:
                continue

            self._entity_map[entity_type.__name__] = True
            entity_type.__table__.create(self.engine)

        return self.session.query(*entity_types)

    def post(self, *entities):
        """
        Insert new `entities` into the database.

        :param entities: a list of new entities.
        """
        session = self.session

        for entity in entities:
            assert isinstance(entity, Entity), 'Expected an entity based on BaseEntity or a declarative base entity.'

            session.add(entity)

        session.commit()
        session.refresh(entity)

    def get(self, entity_type, key):
        """
        Get an entity of type *entity_type*.

        :param entity_type: the class reference of the entities being searched
        :param key:         the lookup key
        """

        return self.query(entity_type).get(key)

    def get_all(self, entity_type):
        """
        Get all entities of type *entity_type*.

        :param entity_type: the class reference of the entities being searched
        """

        return self.query(entity_type).all()


from contextlib import contextmanager
import re
from bson.objectid import ObjectId
from imagination.loader import Loader
from imagination.decorator.validator import restrict_type
from passerine.db.driver.interface import DriverInterface
from passerine.db.session import Session
from passerine.db.exception import InvalidUrlError, UnknownDriverError

class ManagerFactory(object):
    def __init__(self, protocol_to_driver_map = None):
        self._protocol_to_driver_map = {}
        self._alias_to_url_map       = {}
        self._alias_to_manager_map   = {}
        self._re_url = re.compile('(?P<protocol>[a-zA-Z]+)://(?P<address>.+)')

        p2d_map = (protocol_to_driver_map or self._default_protocol_to_driver_map)

        for protocol in p2d_map:
            self.register(protocol, p2d_map[protocol])

    @property
    def _default_protocol_to_driver_map(self):
        return {
            'mongodb': 'passerine.db.driver.mongodriver.Driver',
            #'riak':    'passerine.db.driver.riakdriver.Driver'
        }

    def register(self, protocol, driver_class):
        self._protocol_to_driver_map[protocol] = driver_class \
            if isinstance(driver_class, type) \
            else Loader(driver_class).package

    def analyze_url(self, url):
        index = 0
        connection_info = {}

        matches = self._re_url.match(url)

        if matches:
            return matches.groupdict()

        raise InvalidUrlError('Invalid URL to {}'.format(url))

    def driver(self, url):
        config = self.analyze_url(url)

        if not config:
            raise UnknownDriverError('Unable to connect to {} due to invalid configuration'.format())

        if config['protocol'] in self._protocol_to_driver_map:
            return self._protocol_to_driver_map[config['protocol']](url)

        raise UnknownDriverError('Unable to connect to {}'.format(url))

    def set(self, alias, url):
        self._alias_to_url_map[alias] = url

    def get(self, alias):
        return self.connect(self._alias_to_url_map[alias], alias)

    def connect(self, url, alias = None):
        if alias in self._alias_to_manager_map:
            return self._alias_to_manager_map[alias]

        driver  = self.driver(url)
        manager = Manager(driver)

        if alias:
            self._alias_to_manager_map[alias] = manager

        return manager

class Manager(object):
    """ Entity Manager

        :param driver: the driver interface
        :type  driver: passerine.db.driver.interface.DriverInterface
    """
    def __init__(self, driver):
        assert isinstance(driver, DriverInterface) or issubclass(driver, DriverInterface), \
            'The given driver must implement DriverInterface, {} given.'.format(driver)
        self._driver      = driver
        self._session_map = {}
        self._driver      = driver

    @property
    def driver(self):
        """ Driver API

        :rtype: passerine.db.driver.interface.DriverInterface
        """
        return self._driver

    @contextmanager
    def session(self, id=None, supervised=False):
        session = self.open_session(id, supervised)
        yield session
        self.close_session(id)

    def open_session(self, id=None, supervised=False):
        """ Open a session

            :param id: the session ID
            :param supervised: the flag to indicate that the opening session
                               will be observed and supervised by the manager.
                               This allows the session to be reused by multiple
                               components. However, it is not **thread-safe**.
                               It is disabled by default.
            :type  supervised: bool
        """
        if not supervised:
            return Session(self.driver)

        if not id:
            id = ObjectId()

        if id in self._session_map:
            return self._session_map[id]

        session = Session(self.driver)

        if supervised:
            self._session_map[id] = session

        return session

    def close_session(self, id):
        """ Close the managed session

            .. warning::

                This method is designed to bypass errors when the given ID is
                unavailable or already closed.
        """
        if id not in self._session_map:
            return

        del self._session_map[id]



"""
Mochi Non-relational Database Library
#####################################

:Author: Juti Noppornpitak <jnopporn@shiroyuki.com>

This library is designed to work like SQLite but be compatible with MongoDB instructions (and PyMongo interfaces).
"""

import codecs
import json

class Mochi(dict):
    def __init__(self, location=None):
        self.__location = location

        with codecs.open(self.__location, 'r', 'utf-8') as fp:
            db_map = json.load(fp)

        for id in db_map:
            self[id] = Database(db_map[id])

class Database(dict):
    def __init__(self, initial_data_map={}):
        for id in initial_data_map:
            self[id] = Collection(initial_data_map[id])

class Collection(dict):
    def insert(self, data_set={}):
        pass

    def remove(self, criteria={}):
        pass

    def update(self, criteria={}, update_instructions={}, upsert=False):
        pass



# -*- coding: utf-8 -*-
import json
import re

class InvalidExpressionError(Exception):
    """ Generic Invalid Expression Error """

class DataObject(object):
    @property
    def in_json(self):
        raise RuntimeError('Unknown data object')

    def __str__(self):
        return str(self.in_json)

    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__, self.__str__())

class ExpressionOperand(object):
    OP_EQ = '='
    OP_NE = '!='
    OP_GE = '>='
    OP_GT = '>'
    OP_LE = '<='
    OP_LT = '<'
    OP_IN = 'in'
    OP_NOT_IN = 'not in'
    OP_SQL_LIKE     = 'like'
    OP_REGEXP_LIKE  = 'rlike'
    OP_INDEX_SEARCH = 'indexed with'

class ExpressionType(object):
    IS_PARAMETER     = 'param'
    IS_PROPERTY_PATH = 'path'
    IS_DATA          = 'data'

class Expression(DataObject):
    """ Query Expression

        :param passerine.db.expression.ExpressionPart left:  the left part
        :param passerine.db.expression.ExpressionPart right: the right part
        :param str operand: the generic operand
    """
    def __init__(self, left, operand, right):
        self.left = left
        self.operand = operand
        self.right = right

    @property
    def in_json(self):
        return {
            'left': self.left,
            'right': self.right,
            'operand': self.operand
        }

    def __str__(self):
        return '{left} {operand} {right}'.format(
            left    = self.left.original,
            right   = self.right.original,
            operand = self.operand
        )

class ExpressionPart(DataObject):
    """ Query Expression

        :param str original: the original query
        :param str kind:     the type of the part
        :param     value:    the parameter value only for a data part
        :param str alias:    the entity alias for a property part or the name of the parameter of a parameter part
    """
    def __init__(self, original, kind, value, alias):
        self.original = original
        self.kind     = kind
        self.value    = value
        self.alias    = alias

    @property
    def in_json(self):
        return {
            'original': self.original,
            'kind':     self.kind,
            'value':    self.value,
            'alias':    self.alias
        }

class ExpressionSet(DataObject):
    """ Representation of Analyzed Expression """
    def __init__(self, expressions):
        self.properties  = {}
        self.parameters  = []
        self.expressions = expressions

    @property
    def in_json(self):
        return {
            'properties':  self.properties,
            'parameters':  self.parameters,
            'expressions': self.expressions
        }

class Criteria(object):
    """ Expression Criteria

        Support operands: =, <=, <, >, >=, in, like (SQL-like string pattern), rlike (Regular-expression pattern), indexed with (only for Riak)
    """

    def __init__(self):
        self._is_updated       = False
        self._sub_expressions  = []
        self._analyzed_map     = None
        self._re_parameter     = re.compile('^:[a-zA-Z0-9_]+$')
        self._re_root_path     = re.compile('^[a-zA-Z][a-zA-Z0-9_]*$')
        self._re_property_path = re.compile('^[a-zA-Z][a-zA-Z0-9_]*(\.[a-zA-Z][a-zA-Z0-9_]*)+$')
        self._re_statement     = re.compile(
            '^\s*(?P<left>.+)\s+(?P<operand>{eq}|{ne}|{ge}|{gt}|{le}|{lt}|{xin}|{xnin}|{like}|{rlike}|{indexed})\s+(?P<right>.+)\s*$'.format(
                eq = ExpressionOperand.OP_EQ,
                ne = ExpressionOperand.OP_NE,
                ge = ExpressionOperand.OP_GE,
                gt = ExpressionOperand.OP_GT,
                le = ExpressionOperand.OP_LE,
                lt = ExpressionOperand.OP_LT,
                xin  = ExpressionOperand.OP_IN,
                xnin = ExpressionOperand.OP_NOT_IN,
                like = ExpressionOperand.OP_SQL_LIKE,
                rlike   = ExpressionOperand.OP_REGEXP_LIKE,
                indexed = ExpressionOperand.OP_INDEX_SEARCH
            ),
            re.IGNORECASE
        )
        self._re_property_path_delimiter = re.compile('\.')

    def get_analyzed_version(self):
        if self._is_updated:
            return self._analyzed_map

        analyzed_expression = ExpressionSet(self._sub_expressions)

        # Scan for all property paths and parameters.
        for se in self._sub_expressions:
            self._scan_for_property_paths_and_parameters(analyzed_expression, se.left)
            self._scan_for_property_paths_and_parameters(analyzed_expression, se.right)

        if not analyzed_expression.properties:
            raise InvalidExpressionError('There must be at least one property path. It is prone to query injection.')

        self._analyzed_map = analyzed_expression

        return self._analyzed_map

    def _scan_for_property_paths_and_parameters(self, analyzed_expression, sub_expression_part):
        """ Search for all property paths and parameters.
        """
        # Search for all referred property paths.
        if sub_expression_part.kind == ExpressionType.IS_PROPERTY_PATH:
            property_path = sub_expression_part.original

            analyzed_expression.properties[property_path] = None

        # Search for all referred property paths.
        if sub_expression_part.kind == ExpressionType.IS_PARAMETER:
            parameter_name = sub_expression_part.original[1:]

            analyzed_expression.parameters.append(parameter_name)

    def expect(self, statement):
        self._is_updated = False

        expr = self._compile(statement)

        self._sub_expressions.append(expr)

    @property
    def _fixed_syntax_operands(self):
        return ('in', 'like', 'rlike', 'indexed with')

    def _compile(self, statement):
        fixed_syntax_operands = self._fixed_syntax_operands

        expr = self._parse(statement)

        try:
            expr.left = self._parse_side(expr.left)
        except InvalidExpressionError as exception:
            raise InvalidExpressionError('The left side of the expression cannot be parsed.')

        try:
            expr.right  = self._parse_side(expr.right)
        except InvalidExpressionError as exception:
            raise InvalidExpressionError('The left side of the expression cannot be parsed.')

        # Validate the syntax on the fixed syntaxes.
        if expr.operand in fixed_syntax_operands:
            if expr.left.kind != ExpressionType.IS_PROPERTY_PATH:
                raise InvalidExpressionError('The property path must be on the left of the operand.')

            if expr.right.kind == ExpressionType.IS_PROPERTY_PATH:
                raise InvalidExpressionError('The property path cannot be on the right of the operand.')

        # If the left side refers to the root path but not for the index search,
        # the method will raise the invalid expression error.
        if expr.left.kind == ExpressionType.IS_PROPERTY_PATH \
            and '.' not in expr.left.original \
            and expr.operand != ExpressionOperand.OP_INDEX_SEARCH:
            raise InvalidExpressionError('The property path to the root entity can only be used by index search.')

        # The property path must be in the expression.
        if expr.left.kind != ExpressionType.IS_PROPERTY_PATH and expr.right.kind != ExpressionType.IS_PROPERTY_PATH:
            raise InvalidExpressionError('The property path must be in the expression.')

        return expr

    def _parse_side(self, sub_statement):
        kind = ExpressionType.IS_DATA

        if self._re_parameter.match(sub_statement):
            kind = ExpressionType.IS_PARAMETER

        elif self._re_property_path.match(sub_statement) or self._re_root_path.match(sub_statement):
            kind = ExpressionType.IS_PROPERTY_PATH

        if kind != ExpressionType.IS_DATA:
            alias = self._re_property_path_delimiter.split(sub_statement)[0]\
                if self._re_property_path_delimiter.search(sub_statement) \
                else sub_statement[1:]

            return self._create_expression_part({
                'original': sub_statement,
                'kind':     kind,
                'value':    None,
                'alias':    alias
            })

        decoded_data = None

        try:
            decoded_data = json.loads(sub_statement)
        except ValueError as exception:
            raise InvalidExpressionError('Unable to decode the data.')

        return self._create_expression_part({
            'original': sub_statement,
            'kind':     kind,
            'value':    decoded_data,
            'alias':    None
        })

    def _create_expression_part(self, parameters):
        return ExpressionPart(**parameters)

    def _parse(self, statement):
        matches = self._re_statement.match(statement)

        if not matches:
            raise InvalidExpressionError('Incomplete statement: {}'.format(statement))

        raw_expr   = matches.groupdict()
        expression = Expression(**raw_expr)

        return expression


from riak import RiakClient
from passerine.db.manager import register_driver as driver
from passerine.db.driver.interface import DriverInterface

@driver('mongodb')
class Driver(DriverInterface):
    def db(self, name):
        return None
    
    def collection(self, name):
        return self._client.bucket(name)
    
    def insert(self, collection_name, data):
        riak_object = self.collection(collection_name).new(data=inserted_data)
        riak_object.store()

        return riak_object.key

    def update(self, criteria_or_id, updated_data):
        # Note: criteria_or_id can only be a key due to backend limitation.
        riak_object = self.get(criteria_or_id)

        if not riak_object:
            raise ObjectNotFoundError('Unable to update the object.')

        for property_name in updated_data:
            riak_object.data[property_name] = updated_data[property_name]

        riak_object.store()

    def delete(self, criteria_or_id):
        # Note: criteria_or_id can only be a key due to backend limitation.
        riak_object = self.get(criteria_or_id)

        if not riak_object:
            return False

        riak_object.delete()

        return True

    def connect(self, config):
        self.client = RiakClient(config)

    def disconnect(self): raise NotImplemented()


# Common / Unexpected Computation
class UnexpectedComputationError(Exception):
    """ Exception used when the code runs mistakenly unexpectedly. """

# Common / Future exception
class FutureFeatureException(Exception):
    """
    Exception used when the future feature is used where it is not properly implemented.
    """

# Common / Invalid input
class InvalidInput(Exception):
    """
    Exception used when the given input is invalid or incompatible to the requirement.
    """

# Common / Object Dictionary
class UnsupportObjectTypeError(Exception):
    """
    Exception used when the unsupported object type is used in an inappropriate place.

    Please note that this is a general exception.
    """

# Decorator / Common / Singleton Decorator
class SingletonInitializationException(Exception):
    """
    This exception is used when the target class contain a special
    attribute `_singleton_instance` not a reference to its own class.
    """

# Dependency-injectable Application
class InvalidConfigurationError(Exception):
    """
    Exception thrown only when the configuration is invalid.
    """

# XML Configuration
class DuplicatedPortError(Exception):
    """
    Exception thrown only when the port config is duplicated within the
    same configuration file.
    """

# Routes
class DuplicatedRouteError(Exception):
    """
    Exception used when the routing pattern is already registered.
    """

class RoutingPatternNotFoundError(Exception):
    """
    Exception used when the routing pattern is not specified in the configuration file.
    """

class RoutingTypeNotFoundError(Exception):
    """
    Exception used when the routing type is not specified in the configuration file.
    """

class UnknownRoutingTypeError(Exception):
    """
    Exception used when the routing type is not unknown.
    """

# Controller Directive
class InvalidControllerDirectiveError(Exception):
    """
    Exception used when the controller directive is incomplete due to missing parameter
    """

# Redirection Directive
class InvalidRedirectionDirectiveError(Exception):
    """
    Exception used when the redirection directive is incomplete because some parameters aren't provided or incompatible.
    """

# Fixtures
class LoadedFixtureException(Exception):
    """ Exception raised when the fixture is loaded. """

# Session
class SessionError(Exception):
    """ Exception thrown when there is an error with session component. """

# Template AbstractRepository and Rendering Service
class RenderingSourceMissingError(Exception):
    """
    Exception used when the rendering source is not set.
    """

class UnsupportedRendererError(Exception):
    """
    Exception thrown when the unsupported renderer is being registered.
    """

class RendererSetupError(Exception):
    """
    Exception thrown when there exists errors during setting up the template.
    """

class RendererNotFoundError(Exception):
    """
    Exception thrown when the unknown template repository is used.
    """

# Services
class UnknownServiceError(Exception):
    """
    Exception thrown when the requested service is unknown or not found.
    """


from passerine.decorator.common import singleton

@singleton
class ArraySerializer(object):
    def __init__(self, max_depth=2):
        self._max_depth = max_depth
        self._primitive_types = []
        self._string_types = []

    def set_max_depth(self, max_depth):
        self._max_depth = max_depth

    def encode(self, data, stack_depth=0):
        if not isinstance(data, object):
            raise TypeError('The provided data must be an object')

        returnee = {}

        for name in dir(data):
            if name[0] == '_':
                continue

            value = data.__getattribute__(name)

            if callable(value):
                continue

            if value and not self._is_primitive_type(value):
                if self._max_depth and stack_depth >= self._max_depth:
                    value = value.encode('utf8', 'ignore')
                else:
                    value = self.encode(value, stack_depth + 1)

            returnee[name] = value

        return returnee

    def _is_primitive_type(self, value):
        if not self._primitive_types:
            self._primitive_types = self.default_primitive_types()

        return type(value) in self._primitive_types
    
    def _is_string(self, value):
        if not self._string_types:
            self._string_types = self.default_string_types()

        return type(value) in self._string_types
    
    def default_string_types(self, value):
        try:
            return [str, unicode]
        except NameError as exception:
            return [str]

    def default_primitive_types(self):
        try:
            return [int, float, str, list, dict, tuple, set, bool, unicode, long]
        except NameError as exception:
            return [int, float, str, list, dict, tuple, set, bool]


"""
:Author: Juti Noppornpitak
"""

import re

from imagination.loader    import Loader
from passerine.centre           import services
from passerine.exception        import LoadedFixtureException

class Fixture(object):
    """
    Foundation of the council

    .. note:: this must be used at most once.

    .. warning:: this class is not tested.
    """
    def __init__(self, repository):
        self.__repository   = repository
        self.__loaded_kinds = {}
        self.__fixture_map  = {}
        self.__graphs       = {}

        self.__re_proxy = re.compile('^proxy/(?P<kind>[^/]+)/(?P<alias>.+)$')

    def set(self, kind, fixtures):
        """
        Define the fixtures.

        :param kind: a string represent the kind
        :type  kind: unicode|str
        :param fixtures: the data dictionary keyed by the alias
        :type  fixtures: dict

        .. code-block:: python

            fixture = Fixture()

            fixture.set(
                'council.security.model.Provider',
                {
                    'ldap': { 'name': 'ldap' }
                }
            )
            fixture.set(
                'council.user.model.User', {
                    'admin': { 'name': 'Juti Noppornpitak' }
                }
            )
            fixture.set(
                'council.security.model.Credential',
                {
                    'shiroyuki': {
                        'login':    'admin',
                        'user':     'proxy/council.user.model.User/admin',
                        'provider': 'proxy/council.security.model.Provider/ldap'
                    }
                }
            )

        """
        self.__fixture_map[kind] = fixtures

    @property
    def db(self):
        return self.__repository

    def load(self):
        for kind in self.__fixture_map.keys():
            self.load_fixtures(kind)

    def load_fixtures(self, kind):
        if kind in self.__loaded_kinds:
            return

        loader   = Loader(kind)
        fixtures = self.__fixture_map[kind]

        self.__loaded_kinds[kind] = loader
        self.__graphs[kind]       = []

        for id, fixture in fixtures.iteritems():
            fixture = self._prepare_fixture(fixture)
            entity  = loader.package(**fixture)

            self.db.post(entity)
            self.db.session.refresh(entity)

            fixtures[id] = entity

            self.__graphs[kind].append(entity)

    def _prepare_fixture(self, fixture):
        for key, value in fixture.iteritems():
            proxy_matches = self.__re_proxy.search(value)\
                if (isinstance(value, unicode) or isinstance(value, str))\
                else None

            if not proxy_matches:
                continue

            settings = proxy_matches.groupdict()

            self.load_fixtures(settings['kind'])

            fixture[key] = self.__fixture_map[settings['kind']][settings['alias']]

        return fixture



# -*- coding: utf-8 -*-
"""
:Author: Juti Noppornpitak

This package contains classes and functions for common use.
"""
import codecs
import hashlib
import logging
import random

from .decorator.common import singleton

@singleton
class LoggerFactory(object):
    def __init__(self, default_level=None):
        self.__default_level = default_level or logging.WARNING

    def set_default_level(self, default_level):
        self.__default_level = default_level

        return self

    def make(self, name, level=None, show_time=True):
        level = level or self.__default_level

        logging_handler = logging.StreamHandler()
        logging_handler.setLevel(level)
        logging_handler.setFormatter(
            logging.Formatter(
                '%(levelname)s %(asctime)s %(name)s: %(message)s'
                if show_time
                else '%(levelname)s %(name)s: %(message)s',
                datefmt='%Y.%m.%d %H:%M:%S %Z'
            )
        )

        logger = logging.getLogger(name)
        logger.addHandler(logging_handler)
        logger.setLevel(level)

        return logger

def get_logger(name, level=None, show_time=True):
    return LoggerFactory.instance().make(name, level, show_time)

@singleton
class Enigma(object):
    """ Hashlib wrapper """

    def hash(self, *data_list):
        """
        Make a hash out of the given ``value``.

        :param `data_list`: the list of the data being hashed.
        :type  `data_list`: list of string
        :return:            the hashed data string
        """

        hash_engine = hashlib.new('sha512')
        data_list   = list(data_list)

        for i in range(len(data_list)):
            data_list[i] = codecs.encode(data_list[i], 'ascii')

        hash_engine.update(b''.join(data_list))

        return hash_engine.hexdigest()

    def random_number(self):
        return random.randint(0,100000000000)

class Finder(object):
    """ File System API Wrapper """

    def read(self, file_path, is_binary=False):
        """
        Read a file from *file_path*.

        By default, read a file normally. If *is_binary* is ``True``, the method will read in binary mode.
        """
        with codecs.open(file_path, 'rb' if is_binary else 'r', 'utf-8') as fp:
            file_content = fp.read()

        fp.close()

        return file_content
